package li260.geometrie;

public class Vecteur {
	private double x,y ;

	public Vecteur(double x, double y) {
		super();
		this.x = x;
		this.y = y;
	}
	
	public Vecteur(Vecteur a, Vecteur b) {
		super();
		this.x = b.x - a.x;
		this.y = b.y - a.y;
	}

	public double getX() {
		return x;
	}

	public void setX(double x) {
		this.x = x;
	}

	public double getY() {
		return y;
	}

	public void setY(double y) {
		this.y = y;
	}

	@Override
	public String toString() {
		return "Vecteur [x=" + x + ", y=" + y + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(x);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(y);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Vecteur other = (Vecteur) obj;
		if (Double.doubleToLongBits(x) != Double.doubleToLongBits(other.x))
			return false;
		if (Double.doubleToLongBits(y) != Double.doubleToLongBits(other.y))
			return false;
		return true;
	}
	
	public Vecteur addition (Vecteur b) {
		Vecteur v = new Vecteur (0,0) ;
		v.x = this.x + b.x ;
		v.y = this.y + b.y ;
		return v ;
	}
	
	public void autoAddition (Vecteur b) {
		this.x += b.x ;
		this.y += b.y ;
	}
	
	public Vecteur soustraction (Vecteur b) {
		Vecteur v = new Vecteur (0,0) ;
		v.x = this.x - b.x ;
		v.y = this.y - b.y ;
		//System.out.println("x = "+v.x+" y = "+v.y+"V ===== "+v);
		return v ;
	}
	
	public void autoSoustraction (Vecteur b) {
		this.x -= b.x ;
		this.y -= b.y ;
	}
	
	public double produitScalaire (Vecteur b){
		return this.x*b.x+this.y*b.y;
	}
	
	public double produitVectoriel (Vecteur b){
		return Math.asin(this.produitScalaire(b)/((b.norme())*(this.norme())));
	}
	
	public Vecteur multParUnScalaire (double k){
		Vecteur v = new Vecteur (0,0);
		v.x = this.x * k ;
		v.y = this.y * k ;
		return v ; 
		
	}
	
	public void autoMultParUnScalaire (double k){
		this.x *= k ;
		this.y *= k ;
	}
	
	public Vecteur rotation (double ongle) {
		Vecteur v = new Vecteur (0,0) ;
		double x = this.x ;
		v.x = this.x * Math.cos(ongle) - this.y* Math.sin(ongle) ;
		v.y = x * Math.sin(ongle) + this.y* Math.cos(ongle) ;
		return v ;
	}
	
	public void autoRotation (double ongle) {
		double x = this.x ;
		this.x = this.x * Math.cos(ongle) - this.y* Math.sin(ongle) ;
		this.y = x * Math.sin(ongle) + this.y* Math.cos(ongle) ; 
	}
	
	public double norme () {
		return Math.sqrt(this.x*this.x+this.y*this.y) ;
	}
	
	public Vecteur clonage () {
		 return new Vecteur (this.x,this.y) ;
	}
	
	public boolean equals (Vecteur b) {
		return ((this.x == b.x) && (this.y == b.y )) ;
		
	}
	
	public boolean signe (Vecteur b) {
		return ((this.x*b.y - this.y*b.x)<0) ;
	}
	
	public void toUnitVec () {
		double d = norme (); 
		
		this.x /= d ;
		this.y /=d ;
	}
	
	public double angle(Vecteur b){
		double res= this.produitScalaire(b)/ (this.norme()*b.norme());
		res = Math.max(Math.min(res, 1), -1);
		return this.signe(b)?Math.acos(res):-1*Math.acos(res);
	}
	
	public Vecteur swap(){
		 return new Vecteur(this.getY(),this.getX());
	}

}
