package li260.graphics;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;



















import javax.imageio.ImageIO;
import javax.swing.JPanel;

import org.lwjgl.LWJGLException;

import li260.circuit.Circuit;
import li260.circuit.CircuitFactoryFromFile;
import li260.circuit.TerrainTools;
import li260.dijkstra.Dijkstra;
import li260.geometrie.Vecteur;
import li260.outils.ClavierListener;
import li260.outils.Controleur;
import li260.radar.Radar;
import li260.radar.RadarClassique;
import li260.radar.RadarDijkstra;
import li260.radar.RadarTram;
import li260.simulation.SimulationImpl;
import li260.strategy.SaveListeCommande;
import li260.strategy.Strategy;
import li260.strategy.StrategyComposite;
import li260.strategy.StrategyManuel;
import li260.strategy.StrategyPoint2;
import li260.strategy.StrategyRadar;
import li260.strategy.StrategyTram;
import li260.swing.IHM3Dlight;
import li260.swing.IHMSwing;
import li260.swing.ObserveurSWING;
import li260.view.observeurs.CircuitObserveur3D;
import li260.view.observeurs.ImageObserver;
import li260.view.observeurs.Observeur3D;
import li260.view.observeurs.RadarObserver;
import li260.view.observeurs.TrajectoireObserver;
import li260.view.observeurs.VoitureObserver;
import li260.view.observeurs.VoitureObserveur3D;
import li260.voiture.Voiture;
import li260.voiture.VoitureException;
import li260.voiture.VoitureFactory;

@SuppressWarnings("serial")
public class Action extends JPanel {

	/* instances qui gre l'ensemble de la course */
	private double[] theta  ;//= {Math.PI/2,Math.PI/3 ,Math.PI/6, Math.PI/12,0,-Math.PI/12 , -Math.PI/6,-Math.PI/3,-Math.PI/2};
	private String filename ;
	private IHMSwing ihms ;
	private IHM3Dlight ihms3D;
	private SimulationImpl sim;
	private ArrayList<Voiture> voitures;
	private Circuit circuit;
	private BufferedImage im;
	private boolean event = false; // autorisation de manipuler la souris pour les point ou modificateur de circuit
	private double dimIm = 0;
	ObserveurSWING imo;
	Observeur3D imo3D;
	private Controleur contr;
	private int mode = 0; // 0 -> mode 2D   et  1 -> mode 3D 
	private int nbv = 0; // nombre de voitures 
	private ArrayList<Vecteur> points;	// liste des points de cible pour la voiture
	private ArrayList<Vecteur> danger;  // liste des points qui dfine  la zone de dang 
	@SuppressWarnings("unused") 
	private ArrayList<Integer> choixStrs;  // liste des strategies choisi pour chaque voiture
	private Graphics2D g; 		  
	Dijkstra d; // algorithme de dijkstra

	public Action(double[] theta) throws VoitureException, IOException, LWJGLException{
		super();
		this.theta = theta;
		this.setLayout(new BorderLayout());
		ihms = new IHMSwing();
		ihms3D = new IHM3Dlight();
		this.add(ihms,BorderLayout.CENTER);
		this.contr = new Controleur(this);
		this.addMouseListener(contr);
		this.addMouseMotionListener(contr);
		File f = new File("gif.jpg");
		BufferedImage img = ImageIO.read(f);
		img = Controleur.changeDim(img,4);
		imo = new ImageObserver(img);
		ihms.add(imo);
	}


	/* creation des voitures */
	public void creationV(){
		/******** Creation des Voitures  *********/
		if(circuit == null) return ;
		voitures=new ArrayList<Voiture>();

		for(int i=0;i<this.nbv;i++){

			voitures.add( VoitureFactory.build(circuit)); // factory static   
		}
		ihms3D.addVoiture(voitures.get(0));

		this.points = new ArrayList<Vecteur>();
		this.danger = new ArrayList<Vecteur>();
	}


	/* creation du circuit */
	public void creationC(){
		/**********  Creation de Circuit   ***********/
		circuit = CircuitFactoryFromFile.build(filename);
		ihms.remove();
		ihms3D.remove();
		this.updateUI(); // actualiser Mon JFrame pour une new Image (Circuit)
		nbv = 0;  // initialisation nombre de voitures 

		ihms.setSize(circuit.getWidth(), circuit.getHeight());
		im = TerrainTools.imageFromCircuit(circuit);

		imo = new ImageObserver(im);
		ihms.add(imo);
		imo3D = new CircuitObserveur3D(circuit);
		ihms3D.add(imo3D);
		this.points = new ArrayList<Vecteur>();
		this.danger = new ArrayList<Vecteur>();
		event = true;
	}


	/*** initiation et instalation des voitures et simulation de la course 
	 * @throws InterruptedException ***/

	public void start() throws VoitureException, IOException, InterruptedException{

		ArrayList<Radar> radars;
		ArrayList<Strategy> strs;



		if(circuit == null ) {
			this.g = (Graphics2D) this.getGraphics();
			g.setColor(Color.blue);
			g.setFont(new Font(" TimesRoman ",Font.BOLD,50));
			g.drawString("C H O I S I S S E Z    U N E    I M A G E  ", 200, 200);
			g.drawString("    D E    C I R C U I T     S V P ", 200, 250);
			return ;
		}


		/********     creation Des Strategys ***********/
		strs= new ArrayList<Strategy>();
		radars = new ArrayList<Radar>();




		StrategyComposite strCompo;
		for(int i=0; i<nbv ;i++){

			if(voitures.get(i).getStrategyv().equals("Strategy Classique")){

				radars.add( new RadarClassique(circuit,voitures.get(i),theta));
				strs.add(new StrategyRadar(radars.get(i),voitures.get(i)));
			}

			else if( voitures.get(i).getStrategyv().equals("Strategy Dijkstra")){
				d = new Dijkstra(circuit);   
				d.optimize();  // recalcule Dijkstra car on peut changer le circuit avec le modificateur
				radars.add( new RadarDijkstra(circuit,voitures.get(i),theta,d));
				strs.add(new StrategyRadar(radars.get(i),voitures.get(i)));
			}
			else if(voitures.get(i).getStrategyv().equals("Strategy Points")){
				radars.add(new RadarClassique(circuit,voitures.get(i),theta));
				strs.add(new StrategyPoint2(circuit,voitures.get(i),points));
			}

			else if(voitures.get(i).getStrategyv().equals("Strategy Composite")){
				d = new Dijkstra(circuit);
				d.optimize();
				strCompo = new StrategyComposite();
				StrategyComposite.genererComposite(circuit, voitures.get(i), points, radars, theta, strCompo,50,d,danger);
				strs.add(strCompo);
			}
			else {
				/* Radar Classique par dfault dans le cas de non reconaissance de la stratgie */
				radars.add(new RadarClassique(circuit,voitures.get(i),theta));
				strs.add(new StrategyRadar(radars.get(i),voitures.get(i)));
			}


		}



		/********* Creation des Observateures **********/

		//Observeurs des Radars 

		for(int i=0;i<nbv;i++)
			ihms.add(new  RadarObserver(voitures.get(i),radars.get(i),Color.white));

		// Observeurs des Voitures

		for(int i=0;i<nbv;i++){
			ihms.add(new VoitureObserver(voitures.get(i)));
			ihms3D.add( new VoitureObserveur3D(voitures.get(i)));
		}

		// Observeurs De trajectoire 

		for(int i=0;i<nbv;i++){
			ihms.add(new TrajectoireObserver(this.im,voitures.get(i),Color.cyan));
		}


		/**********  Simulation  **********/

		if(strs.size() != voitures.size()) {
			System.out.println("\t E R R E U R !!\n\t SIMULATION IMPOSSIBLE !!");
			return;
		}

		sim = new SimulationImpl(voitures,circuit,strs);
		sim.add(ihms);
		sim.add(ihms3D);
		sim.play();
		sim.afficher();
		sim.saveImage();

		/**********   Save Des Scores ********/
		for(int i=0;i<nbv;i++)
			if(voitures.get(i).isgagne() == true)  SaveListeCommande.saveListeCommande(sim.getRecord().get(i).getliste(),"record"+i+".cmd") ;



	}
	/*** FIN de start ***/


	// changer entre 2D et 3D
	public void change_mode(){

		if(mode == 0){
			this.remove(ihms);
			this.add(ihms3D,BorderLayout.CENTER);
			mode = 1;
			this.updateUI(); // mtre a jour la fenetre 
		}
		else{
			this.remove(ihms3D);
			this.add(ihms,BorderLayout.CENTER);
			mode = 0;
			this.updateUI();
		}
	}

	/*** geters and seters methods ***/

	// metre a jour le nom du circuit choisi
	public void setCircuit(String name){
		filename = name;
	}

	// metre a jour le nombre de voiture choisi
	public void setNbVoiture(int a) {
		this.nbv = a;
		System.out.println(this.nbv);
		this.creationV();
	}

	public void setChoixStrs( ArrayList<Integer> a){
		this.choixStrs = a;
	}

	public ArrayList<Voiture> getVoitures() {
		return voitures;
	}

	public void setVoitures(ArrayList<Voiture> voitures) {
		this.voitures = voitures;
	}

	public void setCircuit(Circuit circuit) {
		this.circuit = circuit;
	}

	public BufferedImage getIm() {
		return im;
	}

	public void setIm(BufferedImage im) {
		this.im = im;
	}

	public ArrayList<Vecteur> getPoints() {
		return points;
	}

	public void setPoints(ArrayList<Vecteur> points) {
		this.points = points;
	}

	public Circuit getCircuit() {
		// TODO Auto-generated method stub
		return this.circuit;
	}

	public SimulationImpl getSim() {
		return sim;
	}

	public void setSim(SimulationImpl sim) {
		this.sim = sim;
	}

	public int getNbv() {
		return nbv;
	}

	public ArrayList<Vecteur> getDanger() {
		return danger;
	}

	public void setDanger(ArrayList<Vecteur> danger) {
		this.danger = danger;
	}

	public Dijkstra getD() {
		return d;
	}

	public void setD(Dijkstra d) {
		this.d = d;
	}


	public boolean isEvent() {
		return event;
	}


	public void setEvent(boolean event) {
		this.event = event;
	}


}
